@@ -26,7 +26,7 @@ class FileDownload():
         Please see help(FileDownload) for more info
         """
         self.model_url = 'https://drive.google.com/uc?export=download&id=1k4QdDUidCckbX9bQ6JP-9oZ-dso74i8D'
-        self.model_name = '/pyearth_cnn_model_new_test2.h5'
+        self.model_name = '/pyearth_cnn_model_0724.h5'
         self.earth_url = 'https://dl.google.com/dl/earth/client/current/google-earth-stable_current_amd64.deb'
         self.earth_name = '/google-earth-stable_current_amd64.deb'
         self.current_directory = ''
@@ -40,8 +40,9 @@ class FileDownload():
         Checks if model already exists in current directory. If it does not exist, downloads the
         model to the current directory.
         """
+        fileDir = os.path.dirname(os.path.abspath(__file__))
         # Gets current working directory and appends the target file name to it
-        self.current_directory = os.getcwd() + self.model_name
+        self.current_directory = fileDir + self.model_name
         # Checks if the model file already exists in the current directory, returns if so
         if os.path.isfile(self.current_directory):
             return
@@ -57,6 +58,8 @@ class FileDownload():
         if token:
             params = {'confirm' : token}
             self.response = self.session.get(self.model_url, params=params, stream=True)
+        # Notify user of .h5 model file download
+        print("Downloading PyHand Earth .h5 model file")
         # Opens current directory and saves model to it
         with open(self.current_directory, "wb") as my_file:
             for chunk in self.response.iter_content(self.chunk_size):
@@ -74,9 +77,10 @@ class FileDownload():
         """
         # Uses shutil which to determine if Google Earth is already installed
         if which('google-earth-pro') is not None:
-            return
+            return 
+        fileDir = os.path.dirname(os.path.abspath(__file__))
         # Creates path for new google-earth folder in current working directory
-        self.current_directory = os.getcwd() + '/google-earth'
+        self.current_directory = fileDir + '/google-earth'
         # If google-earth folder does not exist, create it
         if not os.path.exists(self.current_directory):
             os.mkdir(self.current_directory)
@@ -87,6 +91,8 @@ class FileDownload():
             return
         # Sends get request and stores response object
         self.response = requests.get(self.earth_url)
+        # Notify user of Google Earth file download
+        print("Downloading Google Earth .deb package file")
         # Saves Google Earth deb file to current working directory/google-earth
         with open(self.current_directory, "wb") as my_file:
             my_file.write(self.response.content)
@@ -102,6 +108,8 @@ class FileDownload():
         self.current_directory = os.getcwd()
         # Change to google-earth folder
         os.chdir('google-earth')
+        # Notify user of Google Earth installation
+        print("Installing Google Earth")
         # Use dpkg to install Google Earth
         os.system('sudo dpkg -i google-earth-stable*.deb')
         # Return to previous directory
