@@ -110,7 +110,8 @@ class GoogleEarth():
         Toggles sidebar in Google Earth that cause problems when sending commands
         to the Google Earth window.
         """
-        sidebar_coords = self.keyboard_commands.locate_image('./images/clicked_sidebar.png')
+        fileDir = os.path.dirname(os.path.abspath(__file__))
+        sidebar_coords = self.keyboard_commands.locate_image(fileDir+'/images/clicked_sidebar.png')
 
         if sidebar_coords:
             self.keyboard_commands.close_sidebar()
@@ -118,16 +119,18 @@ class GoogleEarth():
     def start_up_tips(self):
         """
         Returns True/False flag to pop-up message to close Start-up Tips Google Earth Pro window.
-        """       
-        start_up_tips_coords = self.keyboard_commands.locate_image('PyHand_Earth/images/close.png')
+        """
+        fileDir = os.path.dirname(os.path.abspath(__file__))
+        start_up_tips_coords = self.keyboard_commands.locate_image(fileDir+'/images/close.png')
 
         if start_up_tips_coords:
             return True
-        else: 
+        else:
             return False
 
     def check_if_fullscreen(self):
-        fullscreen = self.keyboard_commands.locate_image('./images/fullscreen.png')
+        fileDir = os.path.dirname(os.path.abspath(__file__))
+        fullscreen = self.keyboard_commands.locate_image(fileDir+'/images/fullscreen.png')
 
         if fullscreen:
             self.keyboard_commands.click_with_location(fullscreen)
